@@ -22,7 +22,6 @@ use sequoia_openpgp as openpgp;
 use openpgp::{
     Cert,
     Fingerprint,
-    crypto::hash::Digest,
     packet::{Key, key::PublicParts, key::UnspecifiedRole},
     policy::Policy,
     types::HashAlgorithm,
@@ -98,7 +97,7 @@ impl CacheTag {
 
     /// Hashes the output.
     fn hash_output(mut self, output: &[u8]) -> Result<Self> {
-        let mut output_hasher = OUTPUT_HASH_ALGO.context()?;
+        let mut output_hasher = OUTPUT_HASH_ALGO.context()?.for_digest();
         output_hasher.update(&output);
         self.hash = output_hasher.into_digest()?;
         Ok(self)
@@ -149,10 +148,10 @@ pub fn make_context() -> Result<Ctx> {
 pub fn agent_keypair(policy: &dyn Policy,
                      cert: &Option<Cert>,
                      key: &Key<PublicParts, UnspecifiedRole>)
-                     -> openpgp::Result<gnupg::KeyPair>
+                     -> openpgp::Result<sequoia_gpg_agent::KeyPair>
 {
     let agent = gnupg::Context::new()?;
-    let mut s = gnupg::KeyPair::new(&agent, key)?;
+    let mut s = sequoia_gpg_agent::KeyPair::new_for_gnupg_context(&agent, key)?;
     if let Some(cert) = cert.as_ref() {
         if let Ok(vcert) = cert.with_policy(policy, None) {
             s = s.with_cert(&vcert);
